<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * MDB_Simple admin container for maintaining Perm/MDB_Simple
 *
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::MDB class.
 */
require_once 'LiveUser/Admin/Perm/Common.php';
require_once 'LiveUser/Perm/Common.php';
require_once 'MDB.php';

/**
 * Class LiveUser_Admin_Perm_MDB_Simple
 *
 * Simple MDB-based complexity driver for LiveUser.
 *
 * Description:
 * This admin class provides the following functionalities
 * - adding users
 * - removing users
 * - changing user data (auth related: username, pwd, active)
 * - adding rights
 * - removing rights
 * - get all users
 *
 * ATTENTION:
 * This class is only experimental. API may change. Use it at your own risk.
 *
 * @author  Bjrn Kraus <krausbn@php.net>
 * @version $Id: MDB_Simple.php,v 1.5 2003/08/13 19:07:28 arnaud Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Admin_Perm_Container_MDB_Simple extends LiveUser_Admin_Perm_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var    string
     * @access private
     */
    var $dsn = null;

    /**
     * PEAR::MDB connection object.
     *
     * @var    object
     * @access private
     */
    var $dbc = null;

    /**
     * Table prefix
     * Prefix for all db tables the container has.
     *
     * @var    string
     * @access public
     */
    var $prefix = 'liveuser_';

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * The second parameters expects the smae array as you use
     * when instantiating LiveUser::factory()
     *
     * @access protected
     * @param  mixed  Array or PEAR::MDB object.
     * @param  array  configuration array
     * @return void
     * @see    LiveUser::factory()
     */
    function &LiveUser_Admin_Container_Perm_MDB_Simple(&$connectOptions, $conf)
    {
        if (is_object($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            $this->dbc = MDB::connect($connectOptions['dsn']);

            if (!MDB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }

        if (isset($conf['permContainer']['prefix'])) {
            $this->prefix = $conf['permContainer']['prefix'];
        }
    } // end func LiveUser_Admin_Perm_MDB_Simple

    /**
     * Adds a new user to Perm/MDB_Simple.
     *
     * @access  public
     * @param   string   $authId      Auth user ID of the user that should be added.
     * @param   int        $type      User type (constants defined in Perm/Common.php) (optional).
     * @return  mixed                 Users perm ID on success, MDB error if not.
     */
    function addUser($authId, $type = LIVEUSER_USER_TYPE_ID)
    {
        if (!$this->init_ok) {
            return false;
        }

        $permId = $this->_getNewPermId();

        $query = '
            INSERT INTO
                '.$this->prefix.'perm_users
                (user_id, auth_user_id, type, auth_container_name)
            VALUES
                (
                '.$this->dbc->getIntegerValue($permId).',
                '.$this->dbc->getTextValue($authId).',
                '.$this->dbc->getIntegerValue($type).',
                '.$this->dbc->getTextValue('MDB').'
                )';
        $result = $this->dbc->query($query);
        if (MDB::isError($result)) {
            return $result;
        }

        return $permId;
    } // end func addUser

    /**
     * Removes an existing user from Perm/MDB_Simple.
     *
     * @access  public
     * @param   string   $authId   Auth user ID of the user that should be removed.
     * @return  mixed              True on success, MDB error if not.
     */
    function removeUser($authId)
    {
        if (!$this->init_ok) {
            return false;
        }

        $permId = $this->_getPermUserId($authId);

        // Delete user's rights (Perm/MDB)
        $query = '
            DELETE FROM
                '.$this->prefix.'userrights
            WHERE
                user_id = '.$this->dbc->getIntegerValue($permId);
        $result = $this->dbc->query($query);
        if (MDB::isError($result)) {
            return $result;
        }

        // Delete user from perm table (Perm/MDB)
        $query = '
            DELETE FROM
                '.$this->prefix.'perm_users
            WHERE
                user_id = '.$this->dbc->getIntegerValue($permId);
        $result = $this->dbc->query($query);
        if (MDB::isError($result)) {
            return $result;
        }

        return true;
    } // end func removeUser

    /**
     * Adds rights for a single user.
     *
     * @access  public
     * @access  string  $authId  Auth user ID.
     * @param   array   $rights  Array of right IDs to add.
     * @return  mixed            True on success, MDB error if not.
     */
    function addRights($authId, $rights)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Insert rights.
        foreach ($rights as $right) {
            $query = '
                INSERT INTO
                    '.$this->prefix.'userrights
                    (user_id, right_id)
                VALUES
                    (
                    '.$this->dbc->getTextValue($this->_getPermUserId($authId)).',
                    '.$this->dbc->getIntegerValue($right).'
                    )';
            $result = $this->dbc->query($query);
            if (MDB::isError($result)) {
                return $result;
            }
        }

        return true;
    } // end func addRights

    /**
     * Removes rights for a single user.
     *
     * @access  public
     * @access  string  $authId  Auth user ID.
     * @param   array   $rights  Array of right IDs to remove or empty to
     *                           remove all.
     * @return  mixed            True on success, MDB error if not.
     */
    function removeRights($authId, $rights = null)
    {
        if (!$this->init_ok) {
            return false;
        }

        $query = '
            DELETE FROM
                '.$this->prefix.'userrights
            WHERE
                user_id = '.$this->_getPermUserId($authId);

        if (is_array($rights)) {
            $query .= ' AND right_id IN ('.implode(', ', $rights);
        }

        $result = $this->dbc->query($query);
        if (MDB::isError($result)) {
            return $result;
        }

        return true;
    } // end func removeRights

    /**
     * Gets all users with handle, password, authId, permId, currentLogin,
     * lastLogin, isActive and individual rights.
     *
     * The array will look like this:
     * $userData[0]['authId']       = 'wujha433gawefawfwfiuj2ou9823r98h';
     *             ['permId']       = '1';
     *             ['type']         = 1;
     *             ['container']    = 'MDB';
     *             ['rights'][0]    = 1;
     *             ['rights'][1]    = 4;
     *             ['rights'][2]    = 5;
     *
     * @access  public
     * @param   boolean  $withRights  If true the rights for each user will be retrieved.
     * @return  mixed                 Array with user data or MDB error.
     */
    function getAllUsers($withRights = false)
    {
        if (!$this->init_ok) {
            return false;
        }

        // First: Get all data from auth and perm table.
        $query = '
            SELECT
                auth_user_id AS authId,
                user_id AS permId,
                type,
                auth_container_name AS container
            FROM
                '.$this->prefix.'perm_users';
        $types = array('text', 'integer', 'integer', 'text');
        $userData = $this->dbc->queryAll($query, $types, MDB_FETCHMODE_ASSOC, true);
        if (MDB::isError($userData)) {
            return $userData;
        }

        // Now: Check if we should get the rights for each user.
        if ($withRights) {
            $query = '
                SELECT
                     user_id, right_id
                FROM
                    '.$this->prefix.'userrights
                ORDER BY
                    user_id ASC';
            $result = $this->dbc->queryAll($query, array('integer', 'integer'), MDB_FETCHMODE_ASSOC, true, false, true);
            if (MDB::isError($result)) {
                return $result;
            }

            $rights = $result;

            $udCount = count($userData);
            $keys = array_keys($userData);

            for ($i = 0; $i < $udCount; $i++) {
                if (isset($rights[$userData[$keys[$i]]['permId']])) {
                    $userData[$keys[$i]]['rights'] = $rights[$userData[$keys[$i]]['permId']];
                }
            }
        }

        return $userData;
    } // end func getAllUsers

    /**
     * Updates user type.
     *
     * @access  public
     * @param   string  $authId  Auth user ID.
     * @param   int     $type    User type (constants defined in Perm/Common.php).
     * @return  mixed            True on success, MDB error if not.
     */
    function setUserType($authId, $type)
    {
        if (!$this->init_ok) {
            return false;
        }

        $query = '
            UPDATE
                '.$this->prefix.'perm_users
            SET
                type = '.$this->dbc->getIntegerValue($type).'
            WHERE
                auth_user_id = '.$this->dbc->getTextValue($authId);
        $result = $this->dbc->query($query);
        if (MDB::isError($result)) {
            return $result;
        }

        return true;
    } // end func setUserType

    /**
     * Gets the perm ID of a user.
     *
     * @access  privat
     * @access  string  $authId  Auth user ID.
     * @return  mixed            Permission ID or MDB error.
     */
    function _getPermUserId($authId)
    {
        // Store persistent information we allready have.
        static $permIds = array();

        if (!isset($permIds[$authId])) {
            $query = '
                SELECT
                    user_id
                FROM
                    '.$this->prefix.'perm_users
                WHERE
                    auth_user_id = '.$this->dbc->getTextValue($authId);
            $permIds[$authId] = $this->dbc->queryOne($query);
        }

        return $permIds[$authId];
    } // end func _getPermUserId

    /**
     * Gets a new perm ID.
     *
     * @access  privat
     * @return  mixed            Permission ID or MDB error.
     */
    function _getNewPermId()
    {
        $newPermId = $this->dbc->nextId($this->prefix.'perm_users');

        if (MDB::isError($newPermId)) {
            return $result;
        }

        return $newPermId;
    } // end func _getNewPermId
}
?>