<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * Shared class for DB-based permissions
 * administration containers.
 *
 * @package  LiveUser
 */

/**
 * Include parent class
 */
require_once 'LiveUser/Admin/Perm/Common.php';

/**
 * This class provides code to all
 * DB based containers used to maintain
 * the permission database.
 *
 * @version $Id: DB_Common.php,v 1.1 2003/08/18 18:30:38 arnaud Exp $
 * @package LiveUser
 */
class LiveUser_Admin_Perm_DB_Common extends LiveUser_Admin_Perm_Common
{
    /**
     * Associative array of all available languages
     * (short name (i.e. "DE", "EN"...) as key, database id as value)
     *
     * @access private
     * @var    array
     */
    var $_langs = array();

    /**
     * Current language (short name (i.e. "DE", "EN"...) being used
     *
     * @access private
     * @var    string
     */
    var $_language = '';

    /**
     * Gets the perm ID of a user.
     *
     * @access  private
     * @access  string  Auth user ID.
     * @return  mixed   Permission ID or DB error.
     */
    function _getPermUserId($authId)
    {
        // Store persistent information we allready have.
        static $permIds = array();

        if (!isset($permIds[$authId])) {
            $query = '
                SELECT
                    user_id
                FROM
                    ' . $this->prefix . 'perm_users
                WHERE
                    auth_user_id = ' . $this->dbc->quote($authId);

            $permIds[$authId] = $this->dbc->getOne($query);
        }

        return $permIds[$authId];
    } // end func _getPermUserId

    /**
     * Gets a new perm ID.
     *
     * @access  private
     * @return  mixed  Permission ID or DB error.
     */
    function _getNewPermId()
    {
        $newPermId = $this->dbc->nextId($this->prefix . 'perm_users');

        if (DB::isError($newPermId)) {
            return $result;
        }

        return $newPermId;
    } // end func _getNewPermId

    /**
     * Reads all languages from databases and stores them in private variable
     *
     * $this->_langs is filled with this array structure:
     *     two_letter_code => language_id
     *
     * @access private
     * @return mixed boolean or DB Error object
     */
    function _getLanguages()
    {
        if (sizeof($this->_langs) < 1) {
            $this->_langs = $this->dbc->getAssoc('SELECT two_letter_name, language_id FROM ' . $this->prefix . 'languages');
            if (DB::isError($this->_langs)) {
                return $this->_langs;
            };
        };
        return true;
    }

    /**
     * Set current language
     *
     * Returns false if the language is not known
     *
     * @access public
     * @param  integer language short name
     * @return mixed   boolean or DB Error object or false
     */
    function setCurrentLanguage($language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        $this->_language = $language;

        return true;
    }

    /**
     * Get current language
     *
     * @access public
     * @return string name of the current language
     */
    function getCurrentLanguage()
    {
        return $this->_language;
    }

    /**
     * Set current application
     *
     * @param integer $application_id   id of application
     *
     * @access public
     * @return boolean always true
     */
    function setCurrentApplication($application_id)
    {
        $this->_application = $application_id;

        return true;
    }

    /**
     * Get current application
     *
     * @access public
     * @return string name of the current application
     */
    function getCurrentApplication()
    {
        return $this->_application;
    }

    /**
     * Assigns name (and comment) in specified language to a section
     *
     * @access public
     * @param integer id of [section]
     * @param integer type of section
     * @param string  language (two letter code) of name/comment
     * @param string  name of [section]
     * @param string  description of [section]
     * @return mixed boolean or DB Error object
     */
    function addTranslation($section_id, $section_type, $language, $name, $comment = null)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        // Register translation
        $query = 'INSERT INTO
                  ' . $this->prefix . 'translations
                  (section_id, section_type, language_id, name, comment)
                VALUES
                  (
                    ' . (int)$section_id . ',
                    ' . (int)$section_type . ',
                    ' . (int)$this->_langs[$language] . ',
                    ' . $this->dbc->quote($name) . ',
                    ' . (($comment === null) ? 'null' : $this->dbc->quote($comment)) . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // name (and comment) added ...
        return true;
    }

    /**
     * Updates name (and comment) of [section] in specified language
     *
     * @access public
     * @param  integer id of [section]
     * @param  integer type of section
     * @param  string  language (two letter code) of name/comment
     * @param  string  name of [section]
     * @param  string  description of [section]
     * @return mixed boolean or DB Error object
     */
    function updateTranslation($section_id, $section_type, $language, $name, $comment = null)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        // Update translation
        $query = 'UPDATE
                  ' . $this->prefix . 'translations
                SET
                  name    = ' . $this->dbc->quote($group_name) . ',
                  comment = ' . (($group_comment === null) ? 'null' : $this->dbc->quote($group_comment)) . '
                WHERE
                  section_id    = ' . (int)$section_id . ' AND
                  section_type  = ' . (int)$section_type . ' AND
                  language_id = ' . (int)$this->_langs[$language];

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Tranlation name (and comment) updated ...
        return true;
    }

    /**
     * Remove name (and comment) of the [section] in specified language
     *
     * @access public
     * @param  integer id of [section]
     * @param  integer type of section
     * @param  string  language (two letter code) of name/comment
     * @param  boolean recursive delete of all translations
     * @return mixed boolean or DB Error object
     */
    function removeTranslation($section_id, $section_type, $language, $recursive = false)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        // Remove translation
        $query = 'DELETE FROM
                  ' . $this->prefix . 'translations
                WHERE
                  section_id    = ' . (int)$section_id . ' AND
                  section_type  = ' . (int)$section_type;
        if (!$recursive) {
            $query .= ' AND language_id = ' . (int)$this->_langs[$language];
        };

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Translation name (and comment) removed ...
        return true;
    }

    /**
     * Get name (and comment) of the [section] in specified language
     *
     * @access public
     * @param  integer id of [section]
     * @param  integer type of section
     * @return mixed array or DB Error object
     */
    function getTranslation($section_id, $section_type)
    {
        // get translation
        $query = 'SELECT
                  translations.name as name,
                  translations.comment as comment
                FROM
                  ' . $this->prefix . 'translations as translations
                WHERE
                  section_id    = ' . (int)$section_id . ' AND
                  section_type  = ' . (int)$section_type;
        $translation = $this->dbc->getRow($query, DB_FETCHMODE_ASSOC);
        if (DB::isError($result)) {
            return $result;
        };

        // Translation name (and comment) removed ...
        return $translation;
    }

    /**
     * Add a new language
     *
     * @access public
     * @param  string two letter code of language
     * @param  string native name of language
     * @param  string name of language
     * @param  string description of language
     * @return mixed integer (language_id) or DB Error object
     */
    function addLanguage($two_letter_code, $native_name, $language_name, $language_comment = null)
    {
        // Get next language id
        $language_id = $this->dbc->nextId($this->prefix . 'languages');

        if (DB::isError($language_id)) {
            return $language_id;
        };

        // Add language
        $query = 'INSERT INTO
                  ' . $this->prefix . 'languages
                  (language_id, two_letter_name, native_name)
                VALUES
                  (
                    ' . $language_id . ',
                    ' . $this->dbc->quote($two_letter_code) . ',
                    ' . $this->dbc->quote($native_name) . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Insert Language tranlation into Tranlastions table
        $result = $this->addTranslation(
            $language_id,
            LIVEUSER_SECTION_LANGUAGE,
            $this->getCurrentLanguage(),
            $language_name,
            $language_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        // Clear language cache
        unset($this->_langs);

        // Job done ...
        return $language_id;
    }

    /**
     * Remove a language
     *
     * @access public
     * @param  integer language (two letter code)
     * @return mixed   boolean or DB Error object
     */
    function removeLanguage($language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        // Delete language
        $query = 'DELETE FROM
                  ' . $this->prefix . 'languages
                WHERE
                  language_id = ' . (int)$this->_langs[$language];

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete language translations
        $result = $this->removeTranslation($this->_langs[$language], LIVEUSER_SECTION_LANGUAGE, $this->getCurrentLanguage(), true);

        if (DB::isError($result)) {
            return $result;
        };

        // Clear language cache
        unset($this->_langs);

        // Job done ...
        return true;
    }

    /**
     * Update language
     *
     * @access public
     * @param  integer language (two letter code)
     * @param  string  native name of language
     * @param  string  name of language
     * @param  string  description of language
     * @return mixed   boolean or DB Error object
     */
    function updateLanguage($language, $native_name, $language_name, $language_comment = null)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };

        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };

        $query = 'UPDATE
                  ' . $this->prefix . 'languages
                SET
                  native_name     = ' . $this->dbc->quote($native_name) . '
                WHERE
                  language_id     = ' . (int)$this->_langs[$language];

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Update Language tranlation into Tranlastions table
        $result = $this->updateTranslation(
            $this->_langs[$language],
            LIVEUSER_SECTION_LANGUAGE,
            $this->getCurrentLanguage(),
            $language_name,
            $langauge_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        // Clear language cache
        unset($this->_langs);

        // Job done ...
        return true;
    }

    /**
     * Add an application
     *
     * @access public
     * @param  string name of application constant
     * @param  string name of application
     * @param  string description of application
     * @return mixed  integer (application_id) or DB Error object
     */
    function addApplication($define_name, $application_name, $application_comment = null)
    {
        // Get next application id
        $application_id = $this->dbc->nextId($this->prefix . 'applications');
        if (DB::isError($application_id)) {
            return $application_id;
        };

        // Register new application
        $query = 'INSERT INTO
                  ' . $this->prefix . 'applications
                  (application_id, application_define_name, application_comment)
                VALUES
                  (
                    ' . (int)$application_id . ',
                    ' . $this->dbc->quote($define_name) . ',
                    ' . $this->dbc->quote($application_comment) . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Insert Application tranlation into Tranlastions table
        $result = $this->addTranslation(
            $application_id,
            LIVEUSER_SECTION_APPLICATION,
            $this->getCurrentLanguage(),
            $application_name,
            $application_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        return $application_id;
    }

    /**
     * Add an area
     *
     * @access public
     * @param  string id of application
     * @param  string name of area constant
     * @param  string name of area
     * @param  string description of area
     * @return mixed  integer (area_id) or DB Error object
     */
    function addArea($application_id, $define_name, $area_name, $area_comment = null)
    {
        // Get next area id
        $area_id = $this->dbc->nextId($this->prefix . 'areas');

        if (DB::isError($area_id)) {
            return $area_id;
        };

        // Register new area
        $query = 'INSERT INTO
                  ' . $this->prefix . 'areas
                  (area_id, area_define_name, application_id, area_comment)
                VALUES
                  (
                    ' . (int)$area_id . ',
                    ' . $this->dbc->quote($define_name) . ',
                    ' . (int)$application_id . ',
                    ' . $this->dbc->quote($area_comment) . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Insert Area tranlation into Tranlastions table
        $result = $this->addTranslation(
            $area_id,
            LIVEUSER_SECTION_AREA,
            $this->getCurrentLanguage(),
            $area_name,
            $area_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        return $area_id;
    }

    /**
     * Delete an application
     *
     * @access public
     * @param  integer id of application
     * @return mixed   boolean or DB Error object or false
     */
    function removeApplication($application_id)
    {
        if (!is_numeric($application_id)) {
            return false;
        }

        // Get all areas within the application, no matter what language
        $query = '
            SELECT
                area_id
            FROM
            ' . $this->prefix . 'areas
            WHERE
                application_id=' . (int)$application_id;

        $areas = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($areas)) {
            return $areas;
        }

        // Delete all areas within the application
        foreach($areas as $area) {
            $res = $this->removeArea($area['area_id']);
            if (DB::isError($res)) {
                return $res;
            }
        }

        // Delete application translations
        $result = $this->removeTranslation($application_id, LIVEUSER_SECTION_APPLICATION, $this->getCurrentLanguage(), true);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete application itself
        $query = 'DELETE FROM
                  ' . $this->prefix . 'applications
                WHERE
                  application_id = ' . $application_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Delete an area
     *
     * @access public
     * @param  integer id of area
     * @return mixed   boolean or DB Error object or false
     */
    function removeArea($area_id)
    {
        if (!is_numeric($area_id)) {
            return false;
        }

        // Delete area itself
        $query = 'DELETE FROM
                  ' . $this->prefix . 'areas
                WHERE
                  area_id = ' . (int)$area_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete area translations
        $result = $this->removeTranslation($area_id, LIVEUSER_SECTION_AREA, $this->getCurrentLanguage(), true);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete all rights in this area
        $query = 'SELECT
                  right_id
                FROM
                  ' . $this->prefix . 'rights
                WHERE
                  area_id = ' . $area_id;

        $result = $this->dbc->getCol($query);

        if (DB::isError($result)) {
            return $result;
        };

        if (is_array($result)) {
            foreach($result as $right_id) {
                $this->removeRight($right_id);
            };
        };

        // Delete area admins
        $query = '
            DELETE FROM
                ' . $this->prefix . 'area_admin_areas
            WHERE
                area_id=' . $area_id . '
        ';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Update an application
     *
     * @access public
     * @param  integer id of application
     * @param  string  name of application constant
     * @param  string  name of application
     * @param  string  description of application
     * @return mixed   boolean or DB Error object
     */
    function updateApplication($application_id, $define_name, $application_name, $application_comment = null)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'applications
                SET
                  application_define_name = ' . $this->dbc->quote($define_name) . ',
                  application_comment     = ' . $this->dbc->quote($application_comment) . '
                WHERE
                  application_id = ' . (int)$application_id;

        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Update Application tranlation into Tranlastions table
        $result = $this->updateTranslation(
            $application_id,
            LIVEUSER_SECTION_APPLICATION,
            $this->getCurrentLanguage(),
            $application_name,
            $application_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Update an area
     *
     * @access public
     * @param  integer id of area
     * @param  string  name of area constant
     * @param  string  name of area
     * @param  string  description of area
     * @return mixed   boolean or DB Error object or false
     */
    function updateArea($area_id, $define_name, $area_name, $area_comment = null)
    {
        if (!is_numeric($area_id)) {
            return false;
        }

        $query = 'UPDATE
                  ' . $this->prefix . 'areas
                SET
                  area_define_name = ' . $this->dbc->quote($define_name) . ',
                  area_comment     = ' . $this->dbc->quote($area_comment) . '
                WHERE
                  area_id = ' . (int)$area_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Update Area tranlation into Tranlastions table
        $result = $this->updateTranslation(
            $area_id,
            LIVEUSER_SECTION_AREA,
            $this->getCurrentLanguage(),
            $area_name,
            $area_comment
        );
        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Add a right in special area
     *
     * @access public
     * @param  integer id of area
     * @param  string  name of right constant
     * @param  string  name of right
     * @param  string  description of right
     * @return mixed   integer (right_id) or DB Error object
     */
    function addRight($area_id, $define_name, $right_name, $right_comment = null)
    {
        // Get next right id
        $right_id = $this->dbc->nextId($this->prefix . 'rights');

        if (DB::isError($right_id)) {
            return $right_id;
        };

        // Register right
        $query = 'INSERT INTO
                  ' . $this->prefix . 'rights
                  (right_id, area_id, right_define_name, right_comment)
                VALUES
                  (
                    ' . (int)$right_id . ',
                    ' . (int)$area_id . ',
                    ' . $this->dbc->quote($define_name) . ',
                    ' . $this->dbc->quote($right_comment) . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Insert Right tranlation into Tranlastions table
        $result = $this->addTranslation(
            $right_id,
            LIVEUSER_SECTION_RIGHT,
            $this->getCurrentLanguage(),
            $right_name,
            $right_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return $right_id;
    }

    /**
     * Delete a right
     *
     * @access public
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function removeRight($right_id)
    {
        // Delete right itself
        $query = 'DELETE FROM
                  ' . $this->prefix . 'rights
                WHERE
                  right_id = ' . (int)$right_id;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        };

        // Delete userright
        $query = 'DELETE FROM
                  ' . $this->prefix . 'userrights
                WHERE
                  right_id = ' . (int)$right_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete group right
        $query = 'DELETE FROM
                  ' . $this->prefix . 'grouprights
                WHERE
                  right_id = ' . (int)$right_id;
        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete right translations
        $result = $this->removeTranslation($right_id, LIVEUSER_SECTION_RIGHT, $this->getCurrentLanguage(), true);

        if (DB::isError($result)) {
            return $result;
        };


        // Job done ...
        return true;

    }

    /**
     * Update a right
     *
     * @access public
     * @param  integer id of right
     * @param  integer id of area
     * @param  string  name of right constant
     * @param  string  name of right
     * @param  string  description of right
     * @return mixed   boolean or DB Error object
     */
    function updateRight($right_id, $area_id, $define_name, $right_name, $right_comment = null)
    {
        $query = 'UPDATE
                  ' . $this->prefix . 'rights
                SET
                  area_id           = ' . (int)$area_id . ',
                  right_define_name = ' . $this->dbc->quote($define_name) . ',
                  right_comment     = ' . $this->dbc->quote($right_comment) . '
                WHERE
                  right_id = ' . (int)$right_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Update Right tranlation into Tranlastions table
        $result = $this->updateTranslation(
            $right_id,
            LIVEUSER_SECTION_RIGHT,
            $this->getCurrentLanguage(),
            $right_name,
            $right_comment
        );

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Add a user
     *
     * @access public
     * @param  string  identifier of the auth container
     * @param  integer user type
     * @return mixed   string (user_id) or DB Error object
     */
    function addUser($authId, $type = LIVEUSER_USER_TYPE_ID)
    {
        if (!$this->init_ok) {
            return false;
        }

        $permId = $this->_getNewPermId();

        $query = '
            INSERT INTO
                ' . $this->prefix . 'perm_users
                (user_id, auth_user_id, type, auth_container_name)
            VALUES
                (
                ' . $permId . ',
                ' . $this->dbc->quote($authId) . ',
                ' . $type . ',
                ' . $this->dbc->quote('DB_Medium') . '
                )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        return $permId;
    }

    /**
     * Delete user
     *
     * @access public
     * @param  string  id of user
     * @return mixed   boolean or DB Error object
     */
    function removeUser($authId)
    {
        if (!$this->init_ok) {
            return false;
        }

        $permId = $this->_getPermUserId($authId);

        // Delete user's rights (Perm/MDB)
        $query = '
            DELETE FROM
                ' . $this->prefix . 'userrights
            WHERE
                user_id = ' . $permId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Delete user from perm table (Perm/MDB)
        $query = '
            DELETE FROM
                ' . $this->prefix . 'perm_users
            WHERE
                user_id = ' . $permId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Delete group assignments
        $query = 'DELETE FROM
                  ' . $this->prefix . 'groupusers
                WHERE
                  user_id = ' . $permId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Delete right assignments
        $query = 'DELETE FROM
                  ' . $this->prefix . 'userrights
                WHERE
                  user_id = ' . $permId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // remove user area admin relation
        $result = $this->removeUserAreaAdmin($authId);

        if (DB::isError($result)) {
            return $result;
        };

        return true;
    }

    /**
     * Grand right to user
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function grantUserRight($authId, $right_id)
    {
        $query = 'INSERT INTO
                  ' . $this->prefix . 'userrights
                  (user_id, right_id, right_level)
                VALUES
                  (
                    ' . $this->_getPermUserId($authId) . ',
                    ' . (int) $right_id . ',
                    1
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Revoke right from user
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function revokeUserRight($authId, $right_id)
    {
        $query = 'DELETE FROM
                  ' . $this->prefix . 'userrights
                WHERE
                  user_id = ' . $this->_getPermUserId($authId) . ' AND
                  right_id = ' . (int) $right_id;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        };

        // Job done ...
        return true;
    }

    /**
     * Get list of all applications
     *
     * This method accepts the following options...
     *  'where_application_id' = [APPLICATION_ID]
     *
     * @access public
     * @param  array an array determining which fields and conditions to use
     * @return mixed array or DB Error object
     */
    function getApplications($options = null)
    {
        $query = 'SELECT
                  applications.application_id as application_id,
                  applications.application_define_name as define_name,
                  translations.name as name,
                  translations.comment as comment
                FROM
                  ' . $this->prefix . 'applications as applications,
                  ' . $this->prefix . 'translations as translations
                WHERE';

        if (isset($options['where_application_id'])
                && is_numeric($options['where_application_id'])) {
            $query .= ' applications.application_id = '
                . (int)$options['where_application_id'] . ' AND ';
        }

        $query .= ' applications.application_id = translations.section_id AND
                  translations.section_type = '
                    . LIVEUSER_SECTION_APPLICATION . ' AND
                  translations.language_id = '
                    . (int)$this->_langs[$this->getCurrentLanguage()] . '
                ORDER BY
                  applications.application_id ASC';

        $applications = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($applications)) {
            return $applications;
        }

        return $applications;
    }


    /**
     * Get list of all areas within a given application
     *
     * This method accepts the following options...
     *  'where_area_id' = [AREA_ID],
     *  'where_application_id' = [APPLICATION_ID],
     *  'with_applications' = [BOOLEAN]
     *
     * @access public
     * @param  array an array determining which fields and conditions to use
     * @return mixed array or DB Error object
     */
    function getAreas($options = null)
    {
        $query = 'SELECT
                  areas.area_id as area_id,
                  areas.application_id as application_id,
                  translations.name as name,
                  translations.comment as comment,
                  areas.area_define_name as define_name
                FROM
                  ' . $this->prefix . 'areas as areas,
                  ' . $this->prefix . 'translations as translations
                WHERE';

        if (isset($options['where_area_id'])
                && is_numeric($options['where_area_id'])) {
                  $query .= ' areas.area_id=' . (int)$options['where_area_id'] . ' AND';
        };

        if (isset($options['where_application_id'])
                && is_numeric($options['where_application_id'])) {
                  $query .= ' areas.application_id=' . (int)$options['where_application_id'] . ' AND';
        };

        $query .= ' areas.area_id = translations.section_id AND
                  translations.section_type = '.LIVEUSER_SECTION_AREA . ' AND
                  translations.language_id = ' . (int)$this->_langs[$this->getCurrentLanguage()] . '
                ORDER BY
                  areas.area_id ASC';

        $areas = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($areas)) {
            return $areas;
        }

        $_areas = array();
        foreach($areas as $key => $value)
        {
            $id = $value['area_id'];
            $_areas[$id] = $value;

            if ($options['with_applications'] == true) {
                $_areas[$id]['application'] = $this->getTranslation($value['application_id'], LIVEUSER_SECTION_APPLICATION);
                if (DB::isError($_areas[$id]['application'])) {
                    return $_areas[$id]['application'];
                }
            };
        }

        return $_areas;
    }

    /**
     * Get list of all languages
     *
     * This method accepts the following options...
     *  'where_language_id' = [LANGUAGE_ID],
     *  'with_translations' = [BOOLEAN]
     *
     * @access public
     * @param  array an array determining which fields and conditions to use
     * @return mixed array or DB Error object
     */
    function getLanguages($options = null)
    {
        $query = 'SELECT
                  languages.language_id as language_id,
                  languages.two_letter_name as two_letter_code,
                  languages.native_name as name
                FROM
                  ' . $this->prefix . 'languages as languages';

        if (isset($options['where_language_id'])
                && is_numeric($options['where_language_id'])) {
            $query .= ' WHERE languages.language_id = ' . (int)$options['where_language_id'];
        };

        $langs = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($langs)) {
            return $langs;
        };

        if (isset($options['with_translations'])
                && $options['with_translations']) {
            $query = 'SELECT
                      translations.native_language_id as native_language_id,
                      languages.two_letter_name as two_letter_code,
                      translations.native_name as translations
                    FROM
                      ' . $this->prefix . 'languages as languages,
                      ' . $this->prefix . 'translations as translations
                    WHERE
                      languages.language_id = translations.language_id';
            $trans = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

            if (DB::isError($trans)) {
                return $trans;
            };
        };

        foreach($langs as $key => $value) {
            unset($langs[$key]);
            $code = $value['two_letter_code'];
            unset($value['two_letter_code']);
            $langs[$code] = $value;

            if ($options['with_translations'] == true) {
                foreach($trans as $translation) {
                    if ($translation['native_language_id'] == $value['language_id']) {
                        $langs[$code]['translations'][$translation['two_letter_code']] = $translation['name'];
                    };
                };
            };
        }
        return $langs;
    }

    /**
     * Get list of all rights
     *
     * This method accepts the following options...
     *  'where_user_id' = [AUTH_USER_ID],
     *  'where_group_id' = [GROUP_ID],
     *  'where_right_id' = [RIGHT_ID],
     *  'where_area_id' = [AREA_ID],
     *  'where_application_id' = [APPLICATION_ID],
     *  'with_areas' = [BOOLEAN],
     *  'with_applications' = [BOOLEAN]
     *
     * @access public
     * @param  array an array determining which fields and conditions to use
     * @return mixed array or DB Error object
     */
    function getRights($options = null)
    {
        $query = 'SELECT
                  rights.right_id as right_id,
                  rights.area_id as area_id,
                  areas.application_id as application_id,';

        if (isset($options['where_user_id'])) {
            $query .= ' userrights.user_id as user_id,';
        }

        if (isset($options['where_group_id'])
                && is_numeric($options['where_area_id'])) {
            $query .= ' grouprights.group_id as group_id,';
        }

        $query .= ' rights.right_define_name as define_name,
                  rights.has_implied as has_implied,
                  rights.has_level as has_level,
                  rights.has_scope as has_scope,
                  translations.name as name,
                  translations.comment as comment
                FROM
                  ' . $this->prefix . 'rights as rights,
                  ' . $this->prefix . 'areas as areas,
                  ' . $this->prefix . 'applications as applications,';

        if (isset($options['where_user_id'])) {
            $query .= ' ' . $this->prefix . 'userrights as userrights,';
        }

        if (isset($options['where_group_id'])
                && is_numeric($options['where_area_id'])) {
            $query .= ' ' . $this->prefix . 'grouprights as grouprights,';
        }

        $query .= ' ' . $this->prefix . 'translations as translations
                WHERE';

        if (isset($options['where_right_id'])
                && is_numeric($options['where_right_id'])) {
            $query .= ' rights.right_id = '
                . (int)$options['where_right_id'] . ' AND';
        };

        if (isset($options['where_area_id'])
                && is_numeric($options['where_area_id'])) {
            $query .= ' rights.area_id = '
                . (int)$options['where_area_id'] . ' AND';
        };

        if (isset($options['where_application_id'])
                && is_numeric($options['where_application_id'])) {
            $query .= ' areas.application_id = '
                . (int)$options['where_application_id'] . ' AND';
        };

        if (isset($options['where_user_id'])
                && is_numeric($options['where_user_id'])) {
            $query .= ' userrights.user_id = '
                . $this->_getPermUserId($options['where_user_id']) . ' AND
                      userrights.right_id = rights.right_id AND';
        };

        if (isset($options['where_group_id'])
                && is_numeric($options['where_area_id'])) {
            $query .= ' grouprights.group_id = '
                . (int)$options['where_group_id'] . ' AND
                      grouprights.right_id = rights.right_id AND';
        };

        $query .= ' rights.area_id = areas.area_id AND
                  rights.right_id = translations.section_id AND
                  translations.section_type = ' . LIVEUSER_SECTION_RIGHT . ' AND
                  translations.language_id = '
                    . (int)($this->_langs[$this->getCurrentLanguage()]) . '
                ORDER BY
                  rights.area_id ASC';

        $rights = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($rights)) {
            return $rights;
        };

        $_rights = array();
        foreach($rights as $key => $value)
        {
            $id = $value['right_id'];
            $_rights[$id] = $value;

            if ($options['with_areas'] == true) {
                // Add area
                $_rights[$id]['area'] =
                    $this->getTranslation($value['area_id'], LIVEUSER_SECTION_AREA);

                if (DB::isError($_rights[$id]['area'])) {
                    return $_rights[$id]['area'];
                };

                if ($options['with_applications'] == true) {
                    // Add application
                    $_rights[$id]['application'] =
                        $this->getTranslation(
                            $value['application_id'],
                            LIVEUSER_SECTION_APPLICATION
                        );

                    if (DB::isError($_rights[$id]['application'])) {
                        return $_rights[$id]['application'];
                    };
                };
            };
        }

        return $_rights;
    }

    /**
     * Make a user an admin of a given area.
     *
     * @access public
     * @param  user identifier
     * @param  area identifier
     * @return mixed  true on success, DB Error object or false
     */
    function setUserAreaAdmin($authId, $area_id)
    {
    	if (!is_numeric($user_id) && !is_numeric($area_id)) {
    	    return false;
    	}

        $permId = $this->__getPermUserId($authId);

        $query = '
            INSERT INTO
                ' . $this->prefix . 'area_admin_areas
                (user_id, area_id)
            VALUES (
                ' . $permId . ', ' . (int)$area_id . '
            )
        ';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        return true;
    }

    /**
     * Remove the privilege of being an admin of a given area.
     *
     * @access public
     * @param  user identifier
     * @param  area identifier
     * @return mixed  true on success, DB Error object or false
     */
    function removeUserAreaAdmin($authId, $area_id)
    {
    	if (!is_numeric($authId) && !is_numeric($area_id)) {
    	    return false;
    	}

        $permId = $this->__getPermUserId($authId);

        $query = '
            DELETE FROM
                ' . $this->prefix . 'area_admin_areas
            WHERE
                user_id=' . $permId . '
            AND
                area_id= ' . (int)$area_id . '
        ';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        return true;
    }
}
?>