<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB container for Authentication
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::DB class.
 */
require_once 'LiveUser/Auth/Common.php';
require_once 'DB.php';

/**
 * Class LiveUser_Auth_Container_DB
 *
 * Description:
 * This is a PEAR::DB backend driver for the LiveUser class.
 * A PEAR::DB connection object can be passed to the constructor to reuse an
 * existing connection. Alternatively, a DSN can be passed to open a new one.
 *
 * Requirements:
 * - File "LoginManager.php" (contains the parent class "LiveUser")
 * - Array of connection options or a PEAR::DB connection object must be
 *   passed to the constructor.
 *   Example: array('dsn'                   => 'mysql://user:pass@host/db_name',
 *                  'connection             => &$conn, # PEAR::DB connection object
 *                  'loginTimeout'          => 0,
 *                  'allowDuplicateHandles' => 1);
 *
 * @author  Markus Wolff <wolff@21st.de>
 * @version $Id: DB.php,v 1.28 2003/07/16 20:29:23 krausbn Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Auth_Container_DB extends LiveUser_Auth_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused)
     * @var string
     * @access private
     */
    var $dsn = NULL;

    /**
     * PEAR::DB connection object
     * @var object
     * @access private
     */
    var $dbc = null;

    /**
     * Auth table
     * Table where the auth data is stored.
     * @var string
     * @access public
     */
    var $authTable = 'liveuser_users';

    /**
     * Columns of the auth table.
     * Associative array with the names of the auth table columns.
     * The 'userId', 'handle' and 'password' fields have to be set.
     * 'lastLogin' and 'isActive' are optional.
     * It doesn't make sense to set only one of the time columns without the
     * other.
     *
     * @var array
     * @access public
     */
    var $authTableCols = array('userId'       => 'user_id',
                               'handle'       => 'handle',
                               'password'     => 'password',
                               'lastLogin'    => 'lastlogin',
                               'isActive'     => 'is_active');

    /**
     * Indicates if backend module initialized correctly. If yes,
     * true, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     * @var boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * LiveUser_Auth_Container_DB::LiveUser_Auth_Container_DB()
     *
     * Class constructor.
     *
     * @param  mixed
     */
    function &LiveUser_Auth_Container_DB(&$connectOptions)
    {
        if (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            if (isset($connectOptions['connection'])  &&
                    DB::isConnection($connectOptions['connection'])
            ) {
                $this->dbc     = &$connectOptions['connection'];
                $this->init_ok = true;
            } elseif (isset($connectOptions['dsn'])) {
                $this->dsn = $connectOptions['dsn'];
                $this->dbc = DB::connect($connectOptions['dsn']);

                if (!DB::isError($this->dbc)) {
                    $this->init_ok = true;
                }
            }
        }
    }

    /**
     * LiveUser_Auth_Container_DB::unfreeze()
     *
     * Reinitializes database connection after the object had been
     * retrieved from the session
     *
     * @param mixed $connectOptions
     * @access public
     */
    function unfreeze(&$connectOptions)
    {
        $this->LiveUser_Auth_Container_DB($connectOptions);
    }

    /**
     * LiveUser_Auth_Container_DB::freeze()
     *
     * Prepares the object for being stored in a session (properly
     * disconnect from the database, free up resources)
     *
     * @access public
     */
    function freeze()
    {
        if ($this->dsn) {
            $this->dbc->disconnect();
        }
        $this->dbc = NULL;
    }

    /**
     * LiveUser_Auth_Container_DB::updateUserData()
     *
     * Writes current values for user back to the database.
     * This method does nothing in the base class and is supposed to
     * be overridden in subclasses according to the supported backend.
     *
     * @return object DB::Result
     * @access private
     */
    function updateUserData()
    {
        if (!$this->init_ok) {
            return false;
        }

        if (isset($this->authTableCols['lastLogin'])) {
            $sql  = 'UPDATE '.$this->authTable.'
                     SET '.$this->authTableCols['lastLogin'].'='.$this->dbc->quote(date('Y.m.d H:i:s', $this->currentLogin)).'
                     WHERE '.$this->authTableCols['userId'].'='.$this->dbc->quote($this->authUserId);
            $res = $this->dbc->query($sql);
            return $res;
        } else {
            return false;
        }
    }

    /**
     * LiveUser_Auth_Container_DB::readUserData()
     *
     * Reads user_id, password, validity flag, current
     * and last login timestamp from the database
     * If only $handle is given, it will read the data
     * from the first user with that handle and return
     * true on success.
     * If $handle and $passwd are given, it will try to
     * find the first user with both handle and password
     * matching and return true on success (this allows
     * multiple users having the same handle but different
     * passwords - yep, some people want this).
     * If no match is found, false is being returned.
     *
     * @param string $handle
     * @param boolean $passwd
     * @return boolean
     */
    function readUserData($handle, $passwd=false)
    {
        if (!$this->init_ok) {
            return false;
        }
        $success = false;

        $fields = array();
        foreach ($this->authTableCols AS $key => $value) {
            $fields[] = $value . ' AS ' . $key;
        }

        if ($passwd !== false) {
            // If $passwd is set, try to find the first user with the given
            // handle and password.
            $sql    = 'SELECT '.implode(',', $fields).'
                       FROM '.$this->authTable.'
                       WHERE '.$this->authTableCols['handle'].'='.$this->dbc->quote($handle).'
                        AND '.$this->authTableCols['password'].'='.$this->dbc->quote($this->encryptPW($passwd));
        } else {
            // If only $handle is set, try to find the first matching user
            $sql    = 'SELECT '.implode(',', $fields).'
                       FROM  '.$this->authTable.'
                       WHERE '.$this->authTableCols['handle'].'='.$this->dbc->quote($handle);
        }

        // Query database
        $result = $this->dbc->getRow($sql, NULL, DB_FETCHMODE_ASSOC);

        // If a user was found, read data into class variables and set
        // return value to true
        if (!DB::isError($result) && is_array($result)) {
            $udata = $result;

            $this->handle       = $udata['handle'];
            $this->passwd       = $this->decryptPW($udata['password']);
            $this->isActive     = ((!isset($udata['isActive']) OR $udata['isActive']=='Y') ? true : false);
            $this->authUserId   = $udata['userId'];
            $this->lastLogin    = !empty($udata['lastLogin'])?
                                    strtotime($udata['lastLogin']):'';

            $success = true;
        }
        return $success;
    }

    /**
     * LiveUser_Auth_Container_DB::userExists()
     *
     * Helper function that checks if there is a user in
     * the database who's matching the given parameters.
     * If $checkHandle is given and $checkPW is set to
     * false, it only checks if a user with that handle
     * exists. If only $checkPW is given and $checkHandle
     * is set to false, it will check if there exists a
     * user with that password. If both values are set to
     * anything but false, it will find the first user in
     * the database with both values matching.
     * Please note:
     * - If no match was found, the return value is false
     * - If a match was found, the user_id from the database
     *   is being returned
     * Whatever is returned, please keep in mind that this
     * function only searches for the _first_ occurence
     * of the search values in the database. So when you
     * have multiple users with the same handle, only the
     * ID of the first one is returned. Same goes for
     * passwords. Searching for both password and handle
     * should be pretty safe, though - having more than
     * one user with the same handle/password combination
     * in the database would be pretty stupid anyway.
     *
     * @param boolean $checkHandle  The handle (username) to search
     * @param boolean $checkPW      The password to check against
     * @return mixed
     */
    function userExists($checkHandle=false,$checkPW=false)
    {
        if (!$this->init_ok) {
            return false;
        }
        if ($checkHandle !== false && $checkPW === false) {
            // only search for the first user with the given handle
            $sql    = 'SELECT '.$this->authTableCols['userId'].'
                       FROM '.$this->authTable.'
                       WHERE '.$this->authTableCols['handle'].'='.$this->dbc->quote($checkHandle);
        } elseif ($checkHandle === false && $checkPW !== false) {
            // only search for the first user with the given password
            $sql    = 'SELECT '.$this->authTableCols['userId'].'
                       FROM '.$this->authTable.'
                       WHERE '.$this->authTableCols['password'].'='.$this->dbc->quote($this->encryptPW($checkPW));
        } else {
            // check for a user with both handle and password matching
            $sql    = 'SELECT '.$this->authTableCols['userId'].'
                       FROM '.$this->authTable.'
                       WHERE '.$this->authTableCols['handle'].'='.$this->dbc->quote($checkHandle).'
                        AND '.$this->authTableCols['password'].'='.$this->dbc->quote($this->encryptPW($checkPW));
        }
        $result = $this->dbc->getOne($sql);
        if (!DB::isError($result) && $result) {
            return $result;
        } else {
            return false;
        }

    }
}
?>