<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * MDB_Medium container for permission handling
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::MDB class.
 */
require_once 'LiveUser/Perm/Common.php';
require_once 'MDB.php';

/**
 * Class LiveUser_Perm_Container_MDB_Medium
 *
 * Medium MDB-based complexity driver for LiveUser.
 *
 * Description:
 * The MDB_Medium provides the following functionalities
 * - users
 * - groups
 * - grouprights
 * - userrights
 * - AuthAreas
 *
 * @author  Arnaud Limbourg
 * @version $Id: MDB_Medium.php,v 1.3 2003/06/18 09:49:02 lsmith Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Perm_Container_MDB_Medium extends LiveUser_Perm_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var string
     * @access private
     **/
    var $dsn = NULL;

    /**
     * PEAR::MDB connection object.
     *
     * @var object
     * @access private
     **/
    var $dbc = NULL;

    /**
     * Table prefix
     * Prefix for all db tables the container has.
     * @var string
     * @access public
     */
    var $prefix = 'liveuser_';

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var boolean
     * @access public
     **/
    var $init_ok = false;

    /**
     * Constructor
     *
     * @param  mixed    $connectOptions   an array or a PEAR::MDB object
     */
    function &LiveUser_Perm_Container_MDB_Medium(&$connectOptions)
    {
        if (is_object($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            $this->dbc = MDB::connect($connectOptions['dsn']);

            if (!MDB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }
    } // end func LiveUser_Perm_Container_MDB_Medium

    /**
     * Tries to find the user with the given user ID in the permissions
     * container. Will read all permission data and return true on success.
     *
     * @access  public
     * @param   string  $uid  user identifier
     * @return  mixed   true on success or a PEAR_Error object
     */
    function init($uid)
    {
        $success = true;

        $query = '
            SELECT
                LU.user_id AS userid,
                LU.type    AS usertype
            FROM
                '.$this->prefix.'perm_users LU
            WHERE
                auth_user_id='.$this->dbc->getValue('text', $uid);

        $result = $this->dbc->queryRow($query, array('integer', 'integer'), MDB_FETCHMODE_ASSOC);

        if (MDB::isError($result)) {
               return $result;
        }

        $this->permUserId = $result['userid'];
        $this->userType   = $result['usertype'];

        if ($this->userType == LIVEUSER_AREAADMIN_TYPE_ID) {
            // get all areas in which the user is area admin
            $query = '
                SELECT
                    LU.area_id
                FROM
                    '.$this->prefix.'area_admin_areas LU
                WHERE
                    LU.user_id='.$this->dbc->getValue('integer', $this->permUserId);

            $result = $this->dbc->queryCol($query, 'integer');

            if (MDB::isError($result)) {
               return $result;
            }

            $this->areaAdmin = $result;
        }

        $this->readRights();

        return $success;
    } // end func init

    /**
     * Reads the group rights
     * and put them in the array
     *
     * right => 1
     *
     * @access  public
     * @return  mixed   MDB_Error on failure or nothing
     */
    function readGroupRights()
    {
        $query = '
            SELECT
                R.right_id AS rightid,
                GU.group_id AS groupid
            FROM
                '.$this->prefix.'groups AS G
            INNER JOIN
                '.$this->prefix.'groupusers AS GU
            ON
                G.group_id=GU.group_id
            INNER JOIN
                '.$this->prefix.'grouprights AS GR
            ON
                GU.group_id=GR.group_id
            INNER JOIN
                '.$this->prefix.'rights R
            ON
                GR.right_id=R.right_id
            WHERE
                G.is_active='.$this->dbc->getValue('boolean', true).'
            AND
                GU.user_id='.$this->dbc->getValue('integer', $this->permUserId);

        $result = $this->dbc->queryAll($query, array('integer', 'integer'), MDB_FETCHMODE_ASSOC);

        if (MDB::isError($result)) {
            return $result;
        } // if

        $allRights = array();
        if(is_array($result)) {
            foreach ($result as $record) {
                $allRights[$record['rightid']] = 1;
                $this->groupIds = array($record['groupid']);
            }
        }

        $this->groupRights = $allRights;
    } // end func readGroupRights

    /**
     * Reads all rights of current user into an
     * associative array.
     * Group rights and invididual rights are being merged
     * in the process.
     *
     * @access  public
     */
    function readRights()
    {
        // reset rights
        $this->rights = array();
        $this->readUserRights();
        $this->readGroupRights();

        $tmpRights = $this->groupRights;

        // Check if user has individual rights...
        if (is_array($this->userRights)) {
            // Overwrite values from temporary array with values from userrights
            foreach ($this->userRights as $right => $level) {
                    if(isset($tmpRights[$right])) {
                        if($level < 0) {
                            $tmpRights[$right] = $tmpRights[$right] - $level;
                        } else {
                            $tmpRights[$right] = max($tmpRights[$right], $level);
                        }
                    } else {
                        $tmpRights[$right] = $level;
                    }
            }
        } else {
            $this->userRights = false;
        }

        // Strip values from array if level is not greater than zero
        $cRights = array();
        if (isset($tmpRights) && is_array($tmpRights)) {
            foreach ($tmpRights as $right => $level) {
               if ($level > 0) {
                   $cRights[$right] = $level;
               }
            }
        }

        if (sizeof($cRights) > 0) {
            $this->rights = $cRights;
        } else {
            $this->rights = array();
        }
    } // end func readRights

    /**
     * Reads the user rights and put them in an array
     *
     * right => level
     *
     * This level is either 1 or -1
     * i.e. you grant the user the right
     * or you do not allow him to have
     * the right.
     *
     * @access  public
     * @return  mixed   MDB_Error on failure or nothing
     */
    function readUserRights()
    {
        $query = '
            SELECT
                R.right_id AS rightid,
                UR.right_level AS level
            FROM
                '.$this->prefix.'userrights UR
            INNER JOIN
                '.$this->prefix.'rights R
            ON
                UR.right_id=R.right_id
            WHERE
                UR.user_id='.$this->dbc->getValue('integer', $this->permUserId);

        $result = $this->dbc->queryAll($query, array('integer', 'integer'), MDB_FETCHMODE_ORDERED, true);

        if (MDB::isError($result)) {
            return $result;
        } // if

        $this->userRights = $result;
    } // end func readUserRights

    /**
     * Prepares the object for being stored in a session (properly
     * disconnect from the database, free up resources).
     *
     * @access  public
     */
    function freeze()
    {
        if($this->dsn) {
            $this->dbc->disconnect();
        }
        $this->dbc = null;
    } // end func freeze

    /**
     * Reinitializes database connection after the object had been
     * retrieved from the session.
     *
     * @access  public
     * @param   array  $connectOptions  connection options
     */
    function unfreeze(&$connectOptions)
    {
        $this->LiveUser_Perm_Container_MDB_Medium($connectOptions);
    } // end func unfreeze
} // end class LiveUser_Perm_Container_MDB_Medium
?>
