<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * MDB_Simple container for permission handling
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::MDB class.
 */
require_once 'LiveUser/Perm/Common.php';
require_once 'MDB.php';

/**
 * Class LiveUser_Perm_Container_MDB_Simple
 *
 * Simple MDB-based complexity driver for LiveUser.
 *
 * Description:
 * The MDB_Simple provides the following functionalities
 * - users
 * - userrights
 *
 * @author  Bjrn Kraus <krausbn@php.net>
 * @version $Id: MDB_Simple.php,v 1.4 2003/06/27 16:34:24 lsmith Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Perm_Container_MDB_Simple extends LiveUser_Perm_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var    string
     * @access private
     */
    var $dsn = NULL;

    /**
     * PEAR::MDB connection object.
     *
     * @var    object
     * @access private
     */
    var $dbc = NULL;

    /**
     * Table prefix
     * Prefix for all db tables the container has.
     *
     * @var    string
     * @access public
     */
    var $prefix = 'liveuser_';

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * @param  mixed    $connectOptions   Array or PEAR::MDB object.
     * @return void
     */
    function &LiveUser_Perm_Container_MDB_Simple(&$connectOptions)
    {
        if (is_object($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            $this->dbc = MDB::connect($connectOptions['dsn']);

            if (!MDB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }
    } // end func LiveUser_Perm_Container_MDB_Simple

    /**
     * Tries to find the user with the given user ID in the permissions
     * container. Will read all permission data and return true on success.
     *
     * @access  public
     * @param   string  $uid  user identifier
     * @return  mixed   true on success or a PEAR_Error object
     */
    function init($uid)
    {
        $success = true;

        $query = '
            SELECT
                LU.user_id AS userid,
                LU.type    AS usertype
            FROM
                '.$this->prefix.'perm_users LU
            WHERE
                auth_user_id='.$this->dbc->getValue('text', $uid);

        $result = $this->dbc->queryRow($query, array('integer', 'integer'), MDB_FETCHMODE_ASSOC);

        if (MDB::isError($result)) {
            return $result;
        }

        $this->permUserId = $result['userid'];
        $this->userType   = $result['usertype'];

        $this->readRights();

        return $success;
    } // end func init

    /**
     * Reads all rights of current user into an
     * associative array.
     *
     * Right => 1
     *
     * @access  public
     * @return  void
     */
    function readRights()
    {
        $query = '
            SELECT
                R.right_id AS rightid, 1
            FROM
                '.$this->prefix.'userrights UR
            INNER JOIN
                '.$this->prefix.'rights R
            ON
                UR.right_id=R.right_id
            WHERE
                UR.user_id='.$this->dbc->getValue('integer', $this->permUserId);

        $result = $this->dbc->queryAll($query, array('integer', 'integer'), MDB_FETCHMODE_ORDERED, true);

        if (MDB::isError($result)) {
            return $result;
        } // if

        $this->rights = $result;
    } // end func readRights

    /**
     * Prepares the object for being stored in a session (properly
     * disconnect from the database, free up resources).
     *
     * @access  public
     * @return  void
     */
    function freeze()
    {
        if($this->dsn) {
            $this->dbc->disconnect();
        }
        $this->dbc = null;
    } // end func freeze

    /**
     * Reinitializes database connection after the object had been
     * retrieved from the session.
     *
     * @access  public
     * @param   array  $connectOptions  connection options
     * @return  void
     */
    function unfreeze(&$connectOptions)
    {
        $this->LiveUser_Perm_Container_MDB_Simple($connectOptions);
    } // end func unfreeze
} // end class LiveUser_Perm_Container_MDB_Simple
?>