<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * XML_Simple permission handling container
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and XML::Tree class.
 */
require_once 'LiveUser/Perm/Common.php';
require_once 'XML/Tree.php';

/**
 * Class LiveUser_Perm_Container_XML_Simple
 *
 * Simple XML-based complexity driver for LiveUser.
 *
 * Description:
 * The XML_Simple provides the following functionalities
 * - users
 * - userrights
 *
 * @author  Bjrn Kraus <krausbn@php.net>
 * @version $Id: XML_Simple.php,v 1.5 2003/07/16 07:44:55 lsmith Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Perm_Container_XML_Simple extends LiveUser_Perm_Common
{
    /**
     * XML file in which the auth data is stored.
     * @var string
     * @access private
     */
    var $file = '';

    /**
     * XML::Tree object.
     *
     * @var    object
     * @access private
     */
    var $tree = null;

    /**
     * XML::Tree object of the user logged in.
     *
     * @var    object
     * @access private
     * @see    readUserData()
     */
    var $userObj = null;

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * @param  mixed $connectoptions  connection options
     * @return void
     */
    function &LiveUser_Perm_Container_XML_Simple(&$connectOptions)
    {
      if (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            if (!is_file($this->file)) {
                if (is_file(getenv('DOCUMENT_ROOT') . $this->file)) {
                    $this->file = getenv('DOCUMENT_ROOT') . $this->file;
                } else {
                    return PEAR::raiseError(null, LIVEUSER_MISSING_DEPS,
                        "Perm initialisation failed. Can't find xml file.", 'LiveUser', true);
                }
            }
            if ($this->file) {
                if (class_exists('XML_Tree')) {
                    $tree =& new XML_Tree($this->file);
                    $err =& $tree->getTreeFromFile();
                    if (PEAR::isError($err)) {
                        return $err;
                    } else {
                        $this->tree = $tree;
                        $this->init_ok = true;
                    }
                } else {
                    $this->_error = true;
                    return PEAR::raiseError(null, LIVEUSER_MISSING_DEPS,
                        "Perm initialisation failed. Can't find XML_Tree class.", 'LiveUser', true);
                }
            } else {
                return PEAR::raiseError(null, LIVEUSER_MISSING_DEPS,
                    "Perm initialisation failed. Can't find xml file.", 'LiveUser', true);
            }
        }
    }

    /**
     * Tries to find the user with the given user ID in the permissions
     * container. Will read all permission data and return true on success.
     *
     * @access  public
     * @param   string  $uid  user identifier
     * @return  mixed   true on success or a PEAR_Error object
     */
    function init($uid)
    {
        if (!$this->init_ok) {
            return false;
        }

        $success = false;
        $nodeIndex = 0;
        $userIndex = 0;

        foreach ($this->tree->root->children as $node) {
            if ($node->name == 'users') {
                foreach ($node->children as $user) {
                    if ($uid == $user->attributes['authUserId']) {
                        $this->permUserId = $user->attributes['userId'];
                        $this->userType   = $user->attributes['type'];
                        $this->userObj    =& $this->tree->root->getElement(array($nodeIndex, $userIndex));
                        $success = true;
                        break;
                    }
                    $userIndex++;
                }
            }
            $nodeIndex++;
        }

        $this->readRights();

        return $success;
    } // end func init

    /**
     * Reads all rights of current user into an
     * associative array.
     *
     * Right => 1
     *
     * @access  public
     * @return void
     */
    function readRights()
    {
        if (!$this->init_ok || !$this->userObj) {
            return false;
        }
        foreach ($this->userObj->children as $node) {
            if ($node->name == 'rights') {
                $tmp = explode(',', $node->content);
                foreach ($tmp as $value) {
                    $this->rights[$value] = 1;
                }
            }
        }
    } // end func readRights

    /**
     * Prepares the object for being stored in a session.
     * If deletes the the XML objects.
     *
     * @access  public
     * @return void
     */
    function freeze()
    {
        $this->tree = null;
        $this->userObj = null;
    } // end func freeze

    /**
     * Reinitializes database connection after the object had been
     * retrieved from the session.
     *
     * @access  public
     * @param   array  $connectOptions  connection options
     * @return void
     */
    function unfreeze(&$connectOptions)
    {
        $this->LiveUser_Perm_Container_XML_Simple($connectOptions);
    } // end func unfreeze
} // end class LiveUser_Perm_Container_XML_Simple
?>
